module Arke
  module Resource
    module Requests
      extend ActiveSupport::Concern

      included do

        class_attribute :response_handlers

        delegate :url, :post, :get, :delete, :put, :handle_response, to: :class

        handle([200, 201, 204]) { |response| deserialize(response.body) }

        handle(404) { raise Errors::NotFound }
        handle(405) { raise Errors::MethodNotAllowed }
        handle(422) { raise Errors::UnprocessableEntity }
        handle(500) { raise Errors::InternalServerError }

      end

      module ClassMethods

        def deserializer(&block)
          @deserializer = block
        end

        def delete(params={})
          HTTParty.delete url(params)
        end

        def post(body={}, params={})
          handle_response(HTTParty.post(url(params.empty? ? body : params), body: body))
        end

        def get(params={})
          handle_response(HTTParty.get url(params))
        end

        def put(body={}, params={})
          handle_response(HTTParty.put(url(params.empty? ? body : params), body: body))
        end

        def handle_response(response)
          self.response_handlers[response.code.to_i].call(response)
        end

        def deserialize(body)
          begin
            @deserializer.call(body)
          rescue => e
            raise Errors::DeserializationError.new(e)
          end
        end

        def handle(code, &block)
          self.response_handlers ||= {}
          raise Arke::Errors::MissingHandlerBlock unless block_given?
          case code
            when String, Integer, Fixnum
              self.response_handlers[code.to_i] = block
              return true
            when Range, Array
              code.to_a.each do |c|
                self.response_handlers[c.to_i] = block
              end
              return true
            else
              raise Errors::InvalidHandler.
                new("#{code.class.name} is an invalid class, please user an Integer, Fixnum, String, Range or Array")
          end
        end

      end

    end
  end
end